use crate::game::*;

use crate::APP_ID;

use gio::Settings;
use gtk::{gio, prelude::*, Entry, Label};
use gtk::{glib::clone, ApplicationWindow, Box};
use rand::prelude::*;
use std::cell::RefCell;
use std::ops::Deref;

fn random_kana(hiragana: bool, katakana: bool) -> Hieroglyph {
    if hiragana && katakana {
        generate_random_hieroglyph(if rand::thread_rng().gen_bool(0.5) {
            Kanas::Katakana
        } else {
            Kanas::Hiragana
        })
    } else {
        generate_random_hieroglyph(if hiragana {
            Kanas::Hiragana
        } else {
            Kanas::Katakana
        })
    }
}

fn ask(ktr: bool, rtk: bool, hiragana: bool, katakana: bool) -> Hieroglyph {
    if ktr && rtk {
        return if rand::thread_rng().gen_bool(0.5) {
            random_kana(hiragana, katakana)
        } else {
            generate_random_hieroglyph(Kanas::Romaji)
        };
    } else if ktr {
        return random_kana(hiragana, katakana);
    } else {
        return generate_random_hieroglyph(Kanas::Romaji);
    }
}

fn generate_question_text(question: &Hieroglyph, exact_kana: &Kanas) -> String {
    return String::from(match question.get_kana() {
        Kanas::Romaji => match exact_kana {
            Kanas::Hiragana => {
                format!("Name corresponding hiragana for {}", question.to_string())
            }
            Kanas::Katakana => {
                format!("Name corresponding katakana for {}", question.to_string())
            }
            _ => {
                panic!("HOW DID YOU GET HERE!?");
            }
        },
        Kanas::Hiragana => {
            format!(
                "Name corresponding romaji for hiragana {}",
                question.to_string()
            )
        }
        Kanas::Katakana => {
            format!(
                "Name corresponding romaji for katakana {}",
                question.to_string()
            )
        }
    });
}

pub fn build_game(_w: &RefCell<ApplicationWindow>) -> Box {
    let game: Box = Box::builder()
        .orientation(gtk::Orientation::Vertical)
        .build();
    let mut question_label: RefCell<Label> =
        RefCell::new(Label::builder().label("placeholder").build());
    let entry: Entry = Entry::builder().placeholder_text("Answer").build();
    let mut counter_label: RefCell<Label> =
        RefCell::new(Label::builder().label("Correct|incorrect: 0|0").build());
    let stats: RefCell<(u32, u32)> = RefCell::new((0, 0));
    let settings: Settings = Settings::new(APP_ID);

    let ktr: bool = settings.boolean("is-ktr-enabled");
    let rtk: bool = settings.boolean("is-rtk-enabled");
    let hiragana: bool = settings.boolean("is-hiragana-enabled");
    let katakana: bool = settings.boolean("is-katakana-enabled");

    let question: RefCell<Hieroglyph> = RefCell::new(ask(ktr, rtk, hiragana, katakana));
    let exact_kana: RefCell<Kanas> = RefCell::new(if hiragana && katakana {
        if rand::thread_rng().gen_bool(0.5) {
            Kanas::Hiragana
        } else {
            Kanas::Katakana
        }
    } else if hiragana {
        Kanas::Hiragana
    } else {
        Kanas::Katakana
    });

    let correct_answer: RefCell<Hieroglyph> = RefCell::new(get_kana_pair_for_hieroglyph(
        match question.borrow().get_kana() {
            Kanas::Romaji => match exact_kana.borrow().deref() {
                Kanas::Hiragana => &Kanas::Hiragana,
                Kanas::Katakana => &Kanas::Katakana,
                _ => panic!("HOW DID YOU GET HERE?!"),
            },
            _ => &Kanas::Romaji,
        },
        &question.borrow(),
    ));

    let binding = generate_question_text(&question.borrow(), exact_kana.borrow().deref());
    let text: &str = &(binding.as_str());
    question_label.borrow_mut().set_label(text);

    let iteration = clone!( @strong question_label, @strong counter_label, @strong stats, @strong exact_kana => move |entry: &Entry| {
        let answer: &String = &entry.text().to_string();
        println!("{} <-> {}? = {}", answer, correct_answer.borrow().to_string(), *answer == correct_answer.borrow().to_string());
        *stats.borrow_mut() = if *answer == correct_answer.borrow().to_string() { (stats.borrow().0 + 1, stats.borrow().1) } else {(stats.borrow().0, stats.borrow().1 + 1)};
        counter_label.borrow_mut().set_label(format!("Correct|Incorrect: {}|{} ({:.2}%)", stats.borrow().0, stats.borrow().1, 100.0 * f64::from(stats.borrow().0) / f64::from(stats.borrow().0 + stats.borrow().1)).as_str());
        *question.borrow_mut() = ask(ktr, rtk, hiragana, katakana);
        *exact_kana.borrow_mut() =
            if hiragana && katakana {
                if rand::thread_rng().gen_bool(0.5) {Kanas::Hiragana} else {Kanas::Katakana}
            } else if hiragana {
                Kanas::Hiragana
            } else {
                Kanas::Katakana
            };
        *correct_answer.borrow_mut() = get_kana_pair_for_hieroglyph(
            match question.borrow().get_kana() {
                Kanas::Romaji => match exact_kana.borrow().deref() {
                    Kanas::Hiragana => &Kanas::Hiragana,
                    Kanas::Katakana => &Kanas::Katakana,
                    _ => panic!("HOW DID YOU GET HERE?!")
                },
                _ => &Kanas::Romaji
            },
            &question.borrow()
        );
        let binding = generate_question_text(&question.borrow(), exact_kana.borrow().deref());
        let text: &str = &(binding.as_str());
        question_label.borrow_mut().set_label(text);
        entry.set_text("");
    });

    entry.connect_activate(iteration);

    game.append(question_label.get_mut());
    game.append(&entry);
    game.append(counter_label.get_mut());

    game
}
