use crate::ui::guessing::build_game;

use crate::APP_ID;

use gio::Settings;
use gtk::{gio, prelude::*, Button};
use gtk::{glib::clone, ApplicationWindow, Box};
use std::cell::{Cell, RefCell};

use crate::labled_switch;

pub fn build_menu(w: &RefCell<ApplicationWindow>) -> Box {
    let menu: Box = Box::builder()
        .orientation(gtk::Orientation::Vertical)
        .halign(gtk::Align::Center)
        .build();

    let settings_container: Box = Box::builder()
        .orientation(gtk::Orientation::Vertical)
        .valign(gtk::Align::Start)
        .halign(gtk::Align::Start)
        .build();

    let gamemode_settings_container: Box = Box::builder()
        .orientation(gtk::Orientation::Vertical)
        .valign(gtk::Align::Start)
        .halign(gtk::Align::Start)
        .build();

    let hiragana_setting_container: Box = Box::builder()
        .orientation(gtk::Orientation::Horizontal)
        .valign(gtk::Align::Center)
        .halign(gtk::Align::Center)
        .build();

    let katakana_setting_container: Box = Box::builder()
        .orientation(gtk::Orientation::Horizontal)
        .valign(gtk::Align::Center)
        .halign(gtk::Align::Center)
        .build();

    let start_button: Button = Button::builder().label("Start!").build();

    let mut settings = Cell::new(Settings::new(APP_ID));

    start_button.connect_clicked(clone!( @strong w => move |_| {
        w.borrow_mut().set_child(Some(&build_game(&w)))
    }));
    //////////////////kana to romaji setting////////////////////////////
    let ktr_switch = labled_switch::build("enable あ->a");
    gamemode_settings_container.append(&ktr_switch.0);

    ////////////////romaji to kana setting///////////////////////////////
    let rtk_switch = labled_switch::build("enable a->あ");
    gamemode_settings_container.append(&rtk_switch.0);
    ////////////////hiragana enable setting/////////////////////////////
    let hiragana_switch = labled_switch::build("enable hiragana");
    hiragana_setting_container.append(&hiragana_switch.0);
    ////////////////katakana enable setting/////////////////////////////
    let katakana_switch = labled_switch::build("enable katakana");
    katakana_setting_container.append(&katakana_switch.0);

    let s = settings.get_mut();
    s.bind("is-ktr-enabled", &ktr_switch.1, "active").build();
    s.bind("is-rtk-enabled", &rtk_switch.1, "active").build();
    s.bind("is-hiragana-enabled", &hiragana_switch.1, "active")
        .build();
    s.bind("is-katakana-enabled", &katakana_switch.1, "active")
        .build();

    settings_container.append(&gamemode_settings_container);
    settings_container.append(&hiragana_setting_container);
    settings_container.append(&katakana_setting_container);

    menu.append(&settings_container);

    menu.append(&start_button);

    menu
}
